@@ -7,7 +7,7 @@ import SimpleITK as sitk
 import json
 import sys
 
-
+# TODO possible bug in norm_bounds when img_type is uint8
 AUTOTUNE = tf.data.experimental.AUTOTUNE
 
 __dataloader_modality__ = ["get", "gen"]
@@ -37,7 +37,6 @@ class DataLoader:
         output_dir=None,
         norm_boundsA=None,
         norm_boundsB=None,
-        # cache_mode="prod",
         extract_only=None,
     ):
         """[summary]
@@ -119,14 +118,28 @@ class DataLoader:
                 self.imgB_paths = self.imgB_paths[:extract_only]
 
             self.is_3D = self.is_3D_data(self.imgA_paths[0])
+            if self.is_3D:
+                self.is_A_RGB = self.is_RGB_data(self.imgA_paths[0])
+                self.is_B_RGB = self.is_RGB_data(self.imgB_paths[0])
+                if self.is_A_RGB or self.is_B_RGB:
+                    self.is_3D = False
+            else:
+                self.is_A_RGB = False
+                self.is_B_RGB = False
             self.imgA_type = self.check_type(self.imgA_paths[0])
             self.imgB_type = self.check_type(self.imgB_paths[0])
             if norm_boundsA is not None:
-                self.norm_boundsA = [float(x) for x in norm_boundsA]
-            else:
+                if norm_boundsA[0] >= norm_boundsA[1]:
+                    raise ValueError(
+                        f"Lower lim for normalization ({norm_boundsA[0]}) must be lower than upper lim ({norm_boundsA[1]})")  # noqa
                 self.norm_boundsA = norm_boundsA
+            else:
+                self.norm_boundsA = None
             if norm_boundsB is not None:
-                self.norm_boundsB = [float(x) for x in norm_boundsB]
+                if norm_boundsB[0] >= norm_boundsB[1]:
+                    raise ValueError(
+                        f"\rLower lim for normalization ({norm_boundsB[0]}) must be lower than upper lim ({norm_boundsB[1]})")  # noqa
+                self.norm_boundsB = norm_boundsB
             else:
                 self.norm_boundsB = norm_boundsB
 
@@ -136,6 +149,8 @@ class DataLoader:
                                 "imgB_label": self.imgB_label,
                                 "imgA_type": self.imgA_type,
                                 "imgB_type": self.imgB_type,
+                                "is_A_RGB": self.is_A_RGB,
+                                "is_B_RGB": self.is_B_RGB,
                                 "norm_boundsA": self.norm_boundsA,
                                 "norm_boundsB": self.norm_boundsB,
                                 }
@@ -143,9 +158,23 @@ class DataLoader:
             output_dir_content = os.listdir(self.output_dir)
             if output_dir_content is not None:
                 if "ds_property.json" not in output_dir_content:
-                    with open(os.path.join(self.output_dir,
+                    # folder is not empty, but property file is missing,
+                    # we need to write it
+                    write_property = True
+                elif len(output_dir_content) == 1 and "ds_property.json" in output_dir_content:  # noqa
+                    # folder contains only an old version of property file,
+                    # we need to overwrite it
+                    write_property = True
+                else:
+                    # every necessary file already exist
+                    write_property = False
+            else:
+                write_property = False
+
+            if write_property:
+                with open(os.path.join(self.output_dir,
                                            "ds_property.json"), 'w') as property_file:  # noqa
-                        json.dump(dataset_property, property_file, indent=2)
+                    json.dump(dataset_property, property_file, indent=2)
 
         elif mode == "get":
             if not os.path.exists(data_dir):
@@ -164,6 +193,8 @@ class DataLoader:
                 self.imgB_label = dataset_property["imgB_label"]
                 self.imgA_type = dataset_property["imgA_type"]
                 self.imgB_type = dataset_property["imgB_type"]
+                self.is_A_RGB = dataset_property["is_A_RGB"]
+                self.is_B_RGB = dataset_property["is_B_RGB"]
                 self.norm_boundsA = dataset_property["norm_boundsA"]
                 self.norm_boundsB = dataset_property["norm_boundsB"]
 
@@ -177,10 +208,12 @@ class DataLoader:
         ds = tf.data.Dataset.zip((self.get_imgs(img_paths=self.imgA_paths,
                                                 img_label=self.imgA_label,
                                                 img_type=self.imgA_type,
+                                                is_RGB=self.is_A_RGB,
                                                 norm_bounds=self.norm_boundsA),
                                   self.get_imgs(img_paths=self.imgB_paths,
                                                 img_label=self.imgB_label,
                                                 img_type=self.imgB_type,
+                                                is_RGB=self.is_B_RGB,
                                                 norm_bounds=self.norm_boundsB)
                                   ))
 
@@ -206,13 +239,18 @@ class DataLoader:
 
         return ds
 
-    def get_imgs(self, img_paths, img_label, img_type, norm_bounds=None):
+    def get_imgs(self,
+                 img_paths,
+                 img_label,
+                 img_type,
+                 is_RGB,
+                 norm_bounds=None):
         """Open image files for one class and store it inside cache.
 
-        This function performs all the (usually) slow reading operations that
+        This function performs all the(usually) slow reading operations that
         is necessary to execute at least the first time. After the first
         execution information are saved inside some cache file inside Cache
-        folder (typically created in your Dataset folder, at the same level of
+        folder(typically created in your Dataset folder, at the same level of
         Images folder). This function detects if cache files are already
         present, and in that case it skips the definition of these files.
         Please take into account that cache files will be as big as your
@@ -220,7 +258,7 @@ class DataLoader:
         bigger amount of time.
 
         Args:
-            img_paths (str): Path to single class images.
+            img_paths(str): Path to single class images.
 
         Returns:
             tf.Data.Dataset: Tensorflow dataset object containing images of one
@@ -246,6 +284,9 @@ class DataLoader:
         if self.is_3D:
             ds = ds.unbatch()
 
+        if is_RGB:
+            ds = ds.map(lambda img: tf.image.rgb_to_grayscale(img))
+
         ds = ds.cache(cache_file)
 
         if not os.path.exists(index_file):
@@ -290,7 +331,7 @@ class DataLoader:
         """Open an image file and convert it to a tensor.
 
         Args:
-            path (tf.Tensor): Tensor containing the path to the file to be
+            path(tf.Tensor): Tensor containing the path to the file to be
                 opened.
 
         Returns:
@@ -302,7 +343,7 @@ class DataLoader:
 
         tensor = tf.convert_to_tensor(image)
 
-        return tf.expand_dims(tensor, axis=-1)
+        return tensor
 
     def _populate_cache(self, ds, cache_file, num_tot):
         print(f"Caching decoded images in {cache_file}...")
@@ -325,7 +366,14 @@ class DataLoader:
         else:
             raise ValueError("Work only with 2D or 3D files.")
 
-        return True
+    @staticmethod
+    def is_RGB_data(path):
+        image = sitk.GetArrayFromImage(sitk.ReadImage(path))
+
+        if image.shape[-1] == 3:
+            return True
+        else:
+            return False
 
     @staticmethod
     def check_type(path):
@@ -346,22 +394,21 @@ class DataLoader:
 
         return imgA, imgB
 
-    # -----------------------------------------------------------
+    # -------------------------------------------------------------------------
     #  Transformations
-    # -----------------------------------------------------------
+    # -------------------------------------------------------------------------
     @ staticmethod
     def norm_with_bounds(image, bounds):
         """Image normalisation. Normalises image in the range defined by lb and
-        ub to fit [0, 1] range."""
-        epsilon = 1e-8
-        lb = bounds[0]
-        ub = bounds[1]
+        ub to fit[0, 1] range."""
+        lb = tf.cast(bounds[0], dtype=image.dtype)
+        ub = tf.cast(bounds[1], dtype=image.dtype)
 
         tf.where(image < lb, lb, image)
         tf.where(image > ub, ub, image)
 
         image = image - lb
-        image /= (ub - lb) + epsilon
+        image /= (ub - lb)
 
         return image
 
